%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sequences(m)
% 
%   Given that rPoly(n,'save') has been called, use the spectral
%   decimation algorithm to generate eigenvalues on VS_n, returning up
%   to 4 arrays:
% 
%     paths   -- successive graph eigenvalues (lambda_m) encountered by
%                spectral decimation
% 
%     words   -- which inverse images of the spectral decimation function
%                were taken
% 
%     eigvals -- the eigenvalues themselves, not neccesarily equal to the
%                the last row of paths since we might refine the
%                eigenvalues with out adding the 1's to paths
% 
%     mults   -- the multiplicities of these eigenvalues
%     
%   More precisely, for each eigenfunction u born on level 0 there will be
%   a column j such that if 
% 
%     [paths words] = sequences(m)
%     path = paths(:,j)
%     word = words(:,j)
% 
%   then
% 
%     - (Delta_k) u == path(k) * u,
%     path(k)       == phi_{word(k)} (path(k-1)).
%     
%   The output of sequences(m) is sorted by eigenvalue. Sequences accepts
%   the following flags
% 
%     'fractal'        -- return fractal eigenvalues (default is graph)
% 
%     '0-series'       -- return only 0-series eigenvalues
% 
%     '4/3-series'     -- return only 4/3-series eigenvalues born at m=0
% 
%     'quiet'          -- don't update paths and words when generating
%                         fractal eigenvalues
%
%     'born-at-0'      -- only include eigenvalues born at m=0
% 
%   See also rPoly, compareWord, continueEigval, small.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [paths,words,eigvals,mults] = sequences(m,varargin)

% we need some eigenvalue info for level N
% these are the polynomials we'll be using 
% (stored as arrays of coefficients)

global rPoly_r rPoly_rho rPoly_bad rPoly_n
n   = rPoly_n;
rho = rPoly_rho;

% by default, we account for the fact that eigenfunctions with eigenvalue
% 4/3 are born at each level, and we start with 0 and 4/3 at m=0
% 0 has multiplicity 1, 4/3 has multiplicity 3
old_paths        = {0,4/3};
old_words        = {0,4/3};
old_mults        = {1,3};

% by default we account for 4/3 series born on each level
create_each_time = 1;

% by default we add ones if things are fractal
add_ones = 1;

% Check for '0-series' flag
if ismember('0-series',varargin)
  old_paths        = {0}; % 0 is our initial eigenvalue
  old_words        = {0}; % 0 is our initial eigenvalue
  create_each_time = 0;   % don't create new 4/3 functions
end

% Check for '4/3-series' flag
if ismember('4/3-series',varargin)
  old_paths        = {4/3}; % 0 is our initial eigenvalue
  old_words        = {4/3}; % 0 is our initial eigenvalue
  create_each_time = 0;    % don't create new 4/3 functions
end

% Check for 'quiet' flag
if ismember('quiet',varargin)
  add_ones = 0;
end

% Check for 'born-at-0' flag
if ismember('born-at-0',varargin)
  create_each_time = 0;
end

% for each level
for j=1:m
  new_paths = {};
  new_words = {};
  new_mults = {};
  % and every path we made in the last pass
  for k=1:length(old_paths)
    % look at the path and word in question
    path  = old_paths{k};
    word  = old_words{k};
    mult  = old_mults{k};
    value = path(length(path));
    % find the images of the last eigenvalue under the spectral
    % decimation function
    triple = continueEigval(value);
    for l=1:length(triple)
      temp = triple(l);
      % if we aren't too close to forbidden eigenvalues
      if ~forbidden([word; l])
        % make a new path and word appending this info to the end
        new_path         = [path;temp];
        new_word         = [word;l];
        new_mult         = mult;
        len              = length(new_paths);
        new_paths{len+1} = new_path;
        new_words{len+1} = new_word;
        new_mults{len+1} = new_mult;
        % disp(new_word');
      end
    end
  end
  % if we're not done the loop, 
  % the paths we just made are the ones we'll extend next
  old_paths = new_paths;
  old_words = new_words;
  old_mults = new_mults;

  % allow for eigenfunctions born on next level with eigenvalue 4/3
  if create_each_time
    len = length(old_paths);
    old_paths{len+1} = 4/3*ones(length(old_paths{1}),1);
    old_words{len+1} = [4/3; zeros(length(old_paths{1})-1,1)];

    % this is the multiplicity of these functions
    old_mults{len+1}=old_mults{len}*(4*n-3)-(4*n-4);
  end
end
        
paths = old_paths;
words = old_words;
mults = old_mults;

% convert everything to matrices
paths = cell2mat(paths);
words = cell2mat(words);
mults = cell2mat(mults);

% look at the last eigenvalues we've encounted in each path
eigenvalues = paths(m+1,:);
[sorted indices] = sort(eigenvalues); % and sort them

% check to see if this agrees with the sorting given by 
% compareWord
need_fix = 0;
%disp('Checking to see if eigenvalue sorting is good enough...');
for j=1:len-1
  if compareWord(words(:,indices(j)),words(:,indices(j+1))) == 1
    need_fix = 1;
    break;
  end
end

% if we need to correct, sort indices according to
% compareWord
if need_fix == 1
  disp('eigenvalue sorting not good enough...');
  disp('sorting manually...');
  len = size(paths,2);
  for j=1:len
    best_index = j;
    best_word  = words(:,indices(best_index));
    for k=j+1:len
      if compareWord(best_word,words(:,indices(k))) == 1
        best_index = k;
        best_word  = words(:,indices(best_index));
      end
    end
    % swap indices(j) with indices(smallest)
    temp                = indices(j);
    indices(j)          = indices(best_index);
    indices(best_index) = temp;
  end
else
  %disp('it was good enough!')
end

% use this information to sort our paths and words
paths = paths(:,indices);
words = words(:,indices);
mults = mults(:,indices);

eigvals = paths(m+1,:);

% now if we've recieved the 'fractal' flag
% we have to clean up our eigenvalues
if ismember('fractal',varargin)
  eigvals = eigvals';
  len     = length(eigvals);
  temp    = small(eigvals);               % apply small
  % if we're updating paths and words, do so
  if add_ones
    paths = [paths; temp'];               % update paths
    words = [words; ones(1,len)];         % update words
  end
  counter = 1;                            % intialize counter
  % we keep continuing with lowest inverse until either
  % 1) things are converging pretty well
  % 2) things screw up because of rounding error
  while 1
    eigvals = small(temp);                % apply small
    err     = max(abs(temp-rho*eigvals)); % redefine error
    if err < 1e-15 || err > 1             % condition on error
      break;
    end
    counter = counter + 1;                % increment counter
    swap    = temp;                       % swap temp and eigvals
    temp    = eigvals;
    eigvals = swap;
    % if we're updating paths and words, do so
    if add_ones
      paths = [paths; eigvals'];          % update paths
      words = [words; ones(1,len)];       % update words
    end
  end
  eigvals = rho^(m+counter)*temp';
end
