%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [inv dups] = invertList(list,m)
% 
%   Given list = makeList(m), invertList creates two arrays, inv and dups.
%   
%   inv
%   
%     The array inv is created such that if x is a base 5 representation of a
%     point in V_m, then inv(x) is the index of this vertex in list.  Note that
%     we may not have
% 
%       list(inv(x)) == x
% 
%     since some vertices have duplicate names.  Note: this overrides the
%     built-in function inv which inverts matrices. 
% 
%   dups
% 
%     As mentioned earlier, base 5 representations of points in V_m are not
%     unique. Dups is an array such that if x is a base 5 representation of a
%     point in V_m, then dups(x) will give the other base-5 representation of
%     this point. Otherwise dups(x) == 0.
% 
%  This program will always be called alongside makeList(m), but has it's own
%  file for debugging and profiling purposes. 
%
%  See also makeList, initialize.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [inv, dups] = invertList(list,m)

% the arrays we'll be filling in
inv  = zeros(5^(m+1),1,'uint32');  
dups = zeros(5^(m+1),1,'uint32');   

% The rules for duplicate names are explained in the README, but
% basically are all of the form "0xx...x == xyy..y". buddies is an array
% such that buddies(x) == y
buddies = uint32([3,4,1,2]);

% powers of 5 that we use to go from arrays to numbers
powers = (5*ones(1,m+1,'uint32')).^uint32([m:-1:0]);

% the length of the array we'll be looping over
len = length(list);

% of course we can fill in a lot of inv by simply requiring
% inv(list(j))=j for j=1:len. This won't deal, however, with duplicate
% names.
for j = 1:len
  x = list(j);
  inv(x)=j;
end

% Now we have to fill in the other points. This requires understanding
% how to create a duplicate name for a given vertex, which is explained
% in the README.
for j = 1:5^(m+1)-1
  if inv(j) == 0
    % j isn't in list, so it must have a duplicate name
    % we construct that duplicate name
    last = mod(j,5);              % last digit of j in base 5
    if last ~= 0                  % things ending in 0 aren't points
      array = num2array(j,m+1);   % we'll set array = digits of duplicate name
      starts_repeating = m+1;     % we'll find where it starts repeating
      buddy = buddies(last);      % "buddy" of last digit 
      % now we replace repeating digits with buddy
      array(m+1) = buddy;
      while array(starts_repeating-1) == last
        array(starts_repeating-1) = buddy;
        starts_repeating = starts_repeating - 1;
      end
      % replace digit before repetition with last digit
      array(starts_repeating-1) = last;  
      % now our array corresponds to the duplicate name, and we can
      % transform this into a number base 5.
      duplicate_name = sum(array.*powers); 
      inv(j) = inv(duplicate_name);
      dups(j) = duplicate_name;
      dups(duplicate_name) = j;
    end
  end
end
