%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% makeList(m)
%   
%   Return a 1-dimensional array which contains the base 5 representations
%   the points in V_m. The array is ordered in the following way
%     
%     [V_0, V_1\V_0, V_2\V_1, ... , V_m\V_{m-1}].
% 
%   Recall that the base 5 representation of a point changes with m. For
%   exmaple, while the first four elements of makeList(0) and makeList(1)
%   represent the same points, the numerical values will be different.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function list = makeList(m)

% a list of powers of 5, used for dealing with numbers base 5
powers = (5*ones(1,m+1,'uint32')).^uint32([m:-1:0]);

% we're dealing with uint32's, so a literal "5" isn't always good enough
five = uint32(5);

len = 3*5^m+1;                 % length of the array we'll create
list = zeros(len,1,'uint32');  % the array we'll return
temp = zeros(1,m+1,'uint32');  % temporary array

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Level 0, our base case. We have only 4 points, each represented
% by a digit 1-4 repeated m+1 times.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

list(1) = sum(1*ones(1,m+1,'uint32').*powers);
list(2) = sum(2*ones(1,m+1,'uint32').*powers);
list(3) = sum(3*ones(1,m+1,'uint32').*powers);
list(4) = sum(4*ones(1,m+1,'uint32').*powers);

% if m was zero, we're done
if m == 0
  return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Level 1, again a special case. The points in V_1\V_0 are of the form
% kjjj....j where k,j = 1,2,3,4 and k~=j.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

position = 5; % the index in list where the previous level starts
for k = 1:4
  for j = 1:4
    if j ~= k 
      % make kjjjjjjjjjjjj
      temp = num2array(list(j),m+1);
      temp(1) = k;
      list(position)=sum(temp.*powers);
      position = position + 1;
    end
  end
end

% if m == 1, we're done
if m == 1
  return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The main loop, same as before but we can use all digits 0-4. 
% we "append" digits to a number using integer arithmetic since it is 
% much faster than using array2num and num2array.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

level_start = 5; % starting point of previous level
level_end = 16;  % ending points of previous level

% for each remaining level
for level = 2:m          
  % start where the last one ended
  position = level_end+1;
  % and append all numbers 0-4 in front of everything in that level
  for k = 0:4                                 
    for j = level_start:level_end
      % append k to list(j,:) 
      list(position) = idivide(list(j),five)+k*five^m;
      position = position + 1;
    end
  end
  % record where the level we just finished begins and ends
  level_start = level_end+1;
  level_end = position-1;
end
