%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% findGapSplit(splits,n)
%
%   Find guaranteed gaps in ratios for VS_n. We use the images of
%   compositions of inverse functions to do this, where there are m
%   functions in a composition. 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function gaps = ratioGapSplit(m,n)

m = m-1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stuff with rPoly
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% find fixed point p
r = rPoly(n,'save'); % NOTE: we'll reassign the variable 'r' later on
global rPoly_deg rPoly_rho
rho = rPoly_rho;
looking_for = sqrt(rho)
deg = rPoly_deg-1;
r_minus_x = r-[zeros(1,deg-1) 1 0];
fixed = roots(r_minus_x);
p = max(fixed);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prepare for linear interpetation of smallN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% number of points in linear interpolation
interp_len = 10000;

% get data points for smallest inverse, small(small_x(i)) = small_y(i)
y_max = small(4/3);                      % highest value we need in small_y
small_y = linspace(0,y_max,interp_len);  % y values
small_x = polyval(r,small_y);            % small_x = R(small_y)
small_x = small_x';
small_y = small_y';

bottom = {};
top    = {};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make a lits of possible compositions to consider
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% if a composition ends in 1, then we just have the images of 0 and 4/3,
% and only if they aren't forbidden.

% words have (m+1) letters from 1:deg, so there a total of 
% deg^m words in base deg
words = (dec2base(0:deg^(m+1)-1,deg)-48+1)';

len = size(words,2);

% these will be the endpoints of our intervals
a = [];
b = [];

% for each word
for j = 1:len
  word = words(:,j)';
  % if it ends in a 1
  if word(length(word)) == 1
    zero_forbidden = forbidden([0 word]);
    one_forbidden  = forbidden([4/3 word]);
    % if [0 word] not forbidden, add image of 0
    if ~zero_forbidden
      zero_image = word2eigval([0 word]);
      if zero_image ~= 0
        a = [a zero_image];
        b = [b zero_image];
      end
    end
    % if [4/3 word] not forbidden, add image of 4/3
    if ~one_forbidden
      one_image = word2eigval([4/3 word]);
      a = [a one_image];
      b = [b one_image];
    end
  % if it doesn't add in a 1, add whole intervals
  else
    a = [a word2eigval([0 word])];
    b = [b word2eigval([p word])];
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now take some limits
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% r = lower endpoints, s = upper endpoints
% NOTE: we're reassigning r. Sorry.
r = min(a,b);
s = max(a,b);

% add in 4/3
r = [r 4/3];
s = [s 4/3];

% create all of the intervals I_{j,k}
% lower = {}; % lower endpoints
% upper = {}; % upper endpoints
len   = length(r);
lower = zeros(1,len*len);
upper = zeros(1,len*len);

counter = 0;
for j = 1:len
  for k = 1:len
    counter = counter + 1;
    next_lower = smallLimitRatio(r(k),s(j),small_x,small_y,interp_len);
    next_upper = smallLimitRatio(s(k),r(j),small_x,small_y,interp_len);
    lower(counter) = next_lower;
    upper(counter) = next_upper;
  end
end

% lower = cell2mat(lower);
% upper = cell2mat(upper);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make a bunch of intervals and test for gaps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% find power k_lower such that rho^-k_lower < min(lower)
% find power k_upper such that rho^k_upper > max(upper)
k_lower = ceil(-log(max(upper))/log(rho));
k_upper = floor(1-log(min(lower))/log(rho));

% we also need to consider the images of these intervals
% under multiplication by rho^k for allowable k
temp1 = lower;
temp2 = upper;
for k=k_lower:k_upper
  lower = [lower rho^k*temp1];
  upper = [upper rho^k*temp2];
end

test = [lower; upper]';
gaps = intervalGap(lower,upper);

% discard any information outside the interval [1,R]
gaps = gaps(:,gaps(2,:) <= rho & gaps(1,:) >= 1);
gappiness = sum(gaps(2,:)-gaps(1,:));
